@@ -9,14 +9,13 @@
  *   3. Macros implementing Easel function argument conventions
  *   4. Macros implementing Easel debugging output conventions
  *   5. Defined constants
- *   6. Basic support for Easel digitized biosequences
- *   7. Using optional compiler attributes
- *   8. Miscellaneous
- *   9. API declarations of easel.c
+ *   6. Using optional compiler attributes
+ *   7. Miscellaneous
+ *   8. API declarations of easel.c
  */
 #ifndef eslEASEL_INCLUDED
 #define eslEASEL_INCLUDED
-#include "esl_config.h"
+#include <esl_config.h>
 
 #include <stdlib.h>
 #include <stdio.h>		/* for FILE */
@@ -126,6 +125,11 @@
 #define eslENOALPHABET    26	/* couldn't guess seq alphabet  */
 #define eslEWRITE         27   	/* write failed (fprintf, etc)  */
 #define eslEINACCURATE    28    /* return val may be inaccurate */
+#define eslEUNSUPPORTEDISA 29   /* function requires an unsupported
+				   instruction set.  Used in stubs
+				   for functions that require a SIMD
+				   ISA that the compiler doesn't 
+				   support. */
 /*::cexcerpt::statuscodes::end::*/
 
 
@@ -146,12 +150,27 @@
  * need a tmp ptr. All ESL_RALLOC() calls can be safely converted
  * to ESL_REALLOC() calls.
  * 
- * The result of malloc(0) is implementation-defined (either NULL or
- * a ptr that may not be dereferenced), a bit of a hole in the C
- * standard. In Easel, we want to avoid having NULL as a valid
- * non-error result of malloc(), because it confuses static analysis
- * tools when they see dereferences of possibly NULL pointers. We
- * therefore treat malloc(0) as an eslEMEM error.
+ * We don't use zero allocations.  The result of malloc(0) is
+ * implementation-defined (either NULL or a ptr that may not be
+ * dereferenced), a bit of a hole in the C standard. In Easel, we want
+ * to avoid having NULL as a valid non-error result of malloc(),
+ * because it confuses static analysis tools when they see
+ * dereferences of possibly NULL pointers. We therefore treat
+ * malloc(0) as an eslEMEM error.
+ *
+ * The `size` argument is >= 0. It can be either signed or unsigned,
+ * but beware of mixed constructs like `(sizeof(foo) * n)`. `sizeof()`
+ * returns unsigned; (unsigned * signed) first converts the signed
+ * operand to unsigned; if the signed operand is negative, the
+ * conversion adds (UINT_MAX+1) modulo (UINT_MAX+1), and a small
+ * negative signed number becomes a ridiculously large unsigned
+ * one. Even when you know n is positive, a -Walloc-size-larger-than
+ * warning in some gcc versions is very aggressively looking for
+ * problems of this sort, where it may assume that your n could have
+ * any value from INT_MIN to -1, generating a false positive compiler
+ * warning. To suppress this warning we typically use a signed cast,
+ * `(ptrdiff_t) sizeof(foo) * n`.
+ * [2023/0804-h3-gcc-malloc-warning]
  */
 /*::cexcerpt::alloc_macros::begin::*/
 #define ESL_ALLOC(p, size) do {\
@@ -214,10 +233,24 @@
 /* Sometimes a shared function API dictates arguments that a function
  * doesn't use, and we want to silence compiler warnings about this.
  * Putting ESL_UNUSED(x) in the function, for an unused argument <x>,
- * should silence the compiler, and should generate a no-op.
+ * satisfies the compiler while generating a no-op.
  */
 #define ESL_UNUSED(x) (void)(sizeof((x)))
 
+/* Sometimes we need a macro's value as a string constant
+ * instead of a number. For example we might have
+ *    #define eslFOO_DEFAULT 42
+ * and we want to use that default value in an esl_getopts OPTIONS
+ * array, where the default needs to be specified as a string constant
+ * "42" not a number 42.  We can use a C preprocessor trick for this,
+ * using the stringize `#` preprocessor operator. It requires two
+ * steps: one to expand the macro, another to convert the expanded
+ * value to a string constant.  You call ESL_STR(eslFOO_DEFAULT). The
+ * ESL_XSTR() macro is just to make the trick work.
+ */
+#define ESL_XSTR(x) #x
+#define ESL_STR(x)  ESL_XSTR(x)
+
 
 /*****************************************************************
  * 4. Macros implementing Easel's debugging output conventions
@@ -303,42 +336,7 @@
 
 
 /*****************************************************************
- * 6. Basic support for Easel's digitized biosequences.
- *****************************************************************/
-
-/* Most of this support is in the alphabet module, but we externalize 
- * some into the easel foundation because ESL_INMAP is used in 
- * sqio, msa modules.
- * 
- * A digital sequence residue (ESL_DSQ) is an unsigned 8-bit type
- * (0..255).  A valid digital residue has a value in the range 0..127
- * (Easel can represent alphabets of up to 128 different characters).
- * Values 128..255 are reserved for flags.
- *
- * An "inmap" is ESL_DSQ[128], or *ESL_DSQ allocated for 128 values;
- * it is a many-to-one construct for mapping 7-bit ASCII chars (in
- * range 0..127) either to new ASCII chars (in the case of raw
- * sequence input in sqio, msa) or to digital codes (in the alphabet
- * module).  Valid mapped values are 0..127; any value in range
- * 128..255 is some kind of flag.
- */
-typedef uint8_t ESL_DSQ;
-#define eslDSQ_SENTINEL 255	/* sentinel bytes 0,L+1 in a dsq */
-#define eslDSQ_ILLEGAL  254	/* input symbol is unmapped and unexpected */
-#define eslDSQ_IGNORED  253     /* input symbol is unmapped and ignored */
-#define eslDSQ_EOL      252	/* input symbol marks end of a line */
-#define eslDSQ_EOD      251     /* input symbol marks end of a seq record */
-
-/* If you try to test sym > 0 && sym <= 127 below, instead of isascii(sym),
- * you'll get a compiler warning for an always-successful test regardless
- * of whether a char is signed or unsigned. So we trust that isascii() is
- * doing the Right Thing.
- */
-#define esl_inmap_IsValid(inmap, sym)  (isascii(sym) && (inmap)[(int)sym] <= 127)
-
-
-/*****************************************************************
- * 7. Using optional compiler attributes
+ * 6. Using optional compiler attributes
  *****************************************************************/
 /* It's convenient to be able to use some optional features of
  * gcc-like compilers, especially when developing, but we have to be
@@ -378,7 +376,7 @@ typedef uint8_t ESL_DSQ;
 
 
 /*****************************************************************
- * 8. Miscellaneous.
+ * 7. Miscellaneous.
  *****************************************************************/
 /* A placeholder for helping w/ portability of filenames/paths.
  * I think, but have not tested, that:
@@ -402,9 +400,10 @@ typedef uint8_t ESL_DSQ;
 #define ESL_MIN(a,b)          (((a)<(b))?(a):(b))
 #define ESL_MAX(a,b)          (((a)>(b))?(a):(b))
 
-static inline float esl_log (double x) { return (x == 0.0 ? -eslINFINITY : log(x));  } /* avoid fp exceptions; log(0) = -inf is fine */
-static inline float esl_logf(float x)  { return (x == 0.0 ? -eslINFINITY : logf(x)); }
-static inline float esl_log2f(float x) { return (x == 0.0 ? -eslINFINITY : eslCONST_LOG2R * logf(x)); }
+static inline float esl_log  (double x) { return (x == 0.0 ? -eslINFINITY : log(x));  } /* avoid fp exceptions; log(0) = -inf is fine */
+static inline float esl_log2 (double x) { return (x == 0.0 ? -eslINFINITY : log2(x)); } 
+static inline float esl_logf (float x)  { return (x == 0.0 ? -eslINFINITY : logf(x)); }
+static inline float esl_log2f(float x)  { return (x == 0.0 ? -eslINFINITY : log2f(x)); }
 
 /* Typedef: <esl_pos_t> 
  * 
@@ -425,7 +424,7 @@ typedef int64_t esl_pos_t;
 
 
 /*****************************************************************
- * 9. The API declarations for easel.c
+ * 8. The API declarations for easel.c
  *****************************************************************/
 
 /* 1. Error handling. */
@@ -438,23 +437,23 @@ extern void esl_nonfatal_handler(int errcode, int use_errno, char *sourcefile, i
 extern void esl_fatal(const char *format, ...) ESL_ATTRIBUTE_NORETURN;
 
 /* 2. Memory allocation/deallocation conventions. */
+extern int  esl_resize(int n, int a, int r);
 extern void esl_free(void *p);
 extern void esl_Free2D(void  **p, int dim1);
 extern void esl_Free3D(void ***p, int dim1, int dim2);
 
 /* 3. Standard banner for Easel miniapplications. */
-extern int  esl_banner    (FILE *fp, char *progname, char *banner);
-extern int  esl_usage     (FILE *fp, char *progname, char *usage);
+extern int  esl_banner    (FILE *fp, const char *progname, char *banner);
+extern int  esl_usage     (FILE *fp, const char *progname, char *usage);
 extern int  esl_dataheader(FILE *fp, ...);
 
 /* 4. Improved replacements for some C library functions */
 extern int  esl_fgets(char **buf, int *n, FILE *fp);
 extern int  esl_fprintf(FILE *fp, const char *format, ...);
 extern int  esl_printf(const char *format, ...);
+extern int  esl_fputc(char c, FILE *fp);
 extern int  esl_strdup(const char *s, int64_t n, char **ret_dup);
 extern int  esl_strcat(char **dest, int64_t ldest, const char *src, int64_t lsrc);
-extern int  esl_strmapcat        (const ESL_DSQ *inmap, char **dest, int64_t *ldest, const char *src, esl_pos_t lsrc);
-extern int  esl_strmapcat_noalloc(const ESL_DSQ *inmap,  char *dest, int64_t *ldest, const char *src, esl_pos_t lsrc);
 extern int  esl_strtok    (char **s, char *delim, char **ret_tok);
 extern int  esl_strtok_adv(char **s, char *delim, char **ret_tok, int *opt_toklen, char *opt_endchar);
 extern int  esl_sprintf (char **ret_s, const char *format, ...);
@@ -496,13 +495,14 @@ extern int  esl_tmpfile_named(char *basename6X, FILE **ret_fp);
 extern int  esl_getcwd(char **ret_cwd);
 
 /* 8. Typed comparison routines. */
-extern int  esl_DCompare   (double a,  double b, double tol);
-extern int  esl_FCompare   (float  a,  float  b, float  tol);
-extern int  esl_DCompareAbs(double a,  double b, double tol);
-extern int  esl_FCompareAbs(float  a,  float  b, float  tol);
-extern int  esl_DCompareNew(double x0, double x, double rtol, double atol);
-extern int  esl_FCompareNew(float  x0, float  x, float  rtol, float  atol);
+extern int  esl_DCompare(double x0, double x, double r_tol, double a_tol);
+extern int  esl_FCompare(float  x0, float  x, float  r_tol, float  a_tol);
 extern int  esl_CCompare(char *s1, char *s2);
+extern int  esl_DCompare_old(double a,  double b, double tol);
+extern int  esl_FCompare_old(float  a,  float  b, float  tol);
+
+/* 9. Other miscellaneous functions */
+extern uint32_t esl_mix3(uint32_t a, uint32_t b, uint32_t c);
 
 #endif /*eslEASEL_INCLUDED*/
 
