@@ -14,7 +14,7 @@
  *   11. Test driver.
  *   12. Examples.
  */
-#include "esl_config.h"
+#include <esl_config.h>
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -654,7 +654,10 @@ esl_msafile_GuessFileFormat(ESL_BUFFER *bf, int *ret_fmtcode, ESL_MSAFILE_FMTDAT
   else // if we haven't guessed so far, try selex.
     {				/* selex parser can handle psiblast too */
       if      (fmt_bysuffix == eslMSAFILE_SELEX) *ret_fmtcode = eslMSAFILE_SELEX;
-      else if (msafile_check_selex(bf) == eslOK) *ret_fmtcode = eslMSAFILE_SELEX;
+      else if (msafile_check_selex(bf) == eslOK) {
+	if (fmt_bysuffix == eslMSAFILE_PSIBLAST) *ret_fmtcode = eslMSAFILE_PSIBLAST;
+	else                                     *ret_fmtcode = eslMSAFILE_SELEX;
+      }
       else    ESL_XFAIL(eslENOFORMAT, errbuf, "couldn't guess alignment input format - doesn't even look like selex");
     }
 
@@ -717,13 +720,13 @@ esl_msafile_EncodeFormat(char *fmtstring)
   if (strcasecmp(fmtstring, "stockholm")   == 0) return eslMSAFILE_STOCKHOLM;
   if (strcasecmp(fmtstring, "pfam")        == 0) return eslMSAFILE_PFAM;
   if (strcasecmp(fmtstring, "a2m")         == 0) return eslMSAFILE_A2M;
-  if (strcasecmp(fmtstring, "phylip")      == 0) return eslMSAFILE_PHYLIP;
-  if (strcasecmp(fmtstring, "phylips")     == 0) return eslMSAFILE_PHYLIPS;
   if (strcasecmp(fmtstring, "psiblast")    == 0) return eslMSAFILE_PSIBLAST;
   if (strcasecmp(fmtstring, "selex")       == 0) return eslMSAFILE_SELEX;
   if (strcasecmp(fmtstring, "afa")         == 0) return eslMSAFILE_AFA;
   if (strcasecmp(fmtstring, "clustal")     == 0) return eslMSAFILE_CLUSTAL;
   if (strcasecmp(fmtstring, "clustallike") == 0) return eslMSAFILE_CLUSTALLIKE;
+  if (strcasecmp(fmtstring, "phylip")      == 0) return eslMSAFILE_PHYLIP;
+  if (strcasecmp(fmtstring, "phylips")     == 0) return eslMSAFILE_PHYLIPS;
   return eslMSAFILE_UNKNOWN;
 }
 
@@ -735,6 +738,9 @@ esl_msafile_EncodeFormat(char *fmtstring)
  *            (<eslMSAFILE_STOCKHOLM>, for example), returns
  *            a string suitable for printing ("Stockholm",
  *            for example).
+ *
+ *            These must be one-word names. We use this call when
+ *            making some space-delimited tabular outputs.
  *            
  * Returns:   a pointer to a static description string.
  * 
@@ -750,14 +756,14 @@ esl_msafile_DecodeFormat(int fmt)
   case eslMSAFILE_UNKNOWN:     return "unknown";
   case eslMSAFILE_STOCKHOLM:   return "Stockholm";
   case eslMSAFILE_PFAM:        return "Pfam";
-  case eslMSAFILE_A2M:         return "UCSC A2M";
+  case eslMSAFILE_A2M:         return "UCSC_A2M";
   case eslMSAFILE_PSIBLAST:    return "PSI-BLAST";
   case eslMSAFILE_SELEX:       return "SELEX";
-  case eslMSAFILE_AFA:         return "aligned FASTA";
+  case eslMSAFILE_AFA:         return "aligned_FASTA";
   case eslMSAFILE_CLUSTAL:     return "Clustal";
   case eslMSAFILE_CLUSTALLIKE: return "Clustal-like";
-  case eslMSAFILE_PHYLIP:      return "PHYLIP (interleaved)";
-  case eslMSAFILE_PHYLIPS:     return "PHYLIP (sequential)";
+  case eslMSAFILE_PHYLIP:      return "PHYLIP(interleaved)";
+  case eslMSAFILE_PHYLIPS:     return "PHYLIP(sequential)";
   default:                     break;
   }
   esl_exception(eslEINVAL, FALSE, __FILE__, __LINE__, "no such msa format code %d\n", fmt);
@@ -1130,11 +1136,18 @@ esl_msafile_Write(FILE *fp, ESL_MSA *msa, int fmt)
  *            alignment format <fmt>, convert it to an <ESL_MSA>.
  *            
  *            For example, 
- *            {\small\begin{verbatim}
- *            esl_msa_CreateFromString("# STOCKHOLM 1.0\n\nseq1 AAAAA\nseq2 AAAAA\n//\n", 
- *                                     eslMSAFILE_STOCKHOLM)
- *            \end{verbatim}}
+ *            ```
+ *              esl_msa_CreateFromString("# STOCKHOLM 1.0\n"
+ *                                       "seq1 AAAAA\n"
+ *                                       "seq2 AAAAA\n"
+ *                                       "//\n", eslMSAFILE_STOCKHOLM);
+ *            ```
  *            creates an ungapped alignment of two AAAAA sequences.
+ *            (Using string concatenation in C99 makes for a cleaner 
+ *            looking multi-line string.)
+ *            
+ *            The <msa> is in text mode. If you want it in digital
+ *            mode, use <esl_msa_Digitize()> on it.
  *
  * Returns:   a pointer to the new <ESL_MSA> on success.
  *
@@ -1237,7 +1250,7 @@ esl_msafile_GetLine(ESL_MSAFILE *afp, char **opt_p, esl_pos_t *opt_n)
  *            
  * Args:      afp  - the open input stream
  *
- * Returns:   <eslOK> on succes
+ * Returns:   <eslOK> on success
  *
  * Throws:    <eslEMEM>, <eslESYS>, <eslEINCONCEIVABLE> if the
  *            <esl_buffer_Set()> call fails.
@@ -1385,7 +1398,7 @@ seq2    ACDEFGHIKLMNPQRSTVWYacdefghiklmnpqrstvwyACDEFGHIKLMNPQRSTVWYacdefghiklmn
  *  (gcov): gcc -g -Wall -fprofile-arcs -ftest-coverage -I. -L. -o esl_msafile_utest -DeslMSAFILE_TESTDRIVE esl_msafile.c -leasel -lm
  * run:     ./esl_msafile_utest
  */
-#include "esl_config.h"
+#include <esl_config.h>
 
 #include <stdio.h>
 
