@@ -15,6 +15,7 @@
 	
 #include "easel.h"
 #include "esl_distance.h"
+#include "esl_dsq.h"
 #include "esl_msa.h"
 #include "esl_msacluster.h"
 #include "esl_msaweight.h"
@@ -27,6 +28,8 @@
 
 
 #include "msamanip.h"
+#include "r2rdepict.h"
+#include "structure.h"
 
 
 static int calculate_Cstats(ESL_MSA *msa, int *ret_maxilen, int *ret_totilen, int *ret_totinum, double *ret_avginum, double *ret_stdinum, double *ret_avgilen,
@@ -34,10 +37,68 @@ static int calculate_Cstats(ESL_MSA *msa, int *ret_maxilen, int *ret_totilen, in
 static int calculate_Xstats(ESL_MSA *msa, int *ret_maxilen, int *ret_totilen, int *ret_totinum, double *ret_avginum, double *ret_stdinum, double *ret_avgilen, 
 			    double *ret_stdilen, double *ret_avgsqlen, double *ret_stdsqlen, int *ret_anclen);
 static int reorder_msa(ESL_MSA *msa, int *order, char *errbuf);
-static int shuffle_tree_substitutions(ESL_RANDOMNESS *r, int aidx, int didx, ESL_DSQ *axa, ESL_DSQ *axd, ESL_MSA *shallmsa, int *usecol, char *errbuf, int verbose);
+static int shuffle_tree_substitutions(ESL_RANDOMNESS *r, int aidx, int didx, ESL_DSQ *axa, ESL_DSQ *axd, ESL_MSA *shallmsa, int *usecol,
+				      char *errbuf, int verbose);
 static int shuffle_tree_substitute_all(ESL_RANDOMNESS *r, int K, int *nsub, int L, ESL_DSQ *ax, ESL_DSQ *new, int *usecol, char *errbuf);
 static int shuffle_tree_substitute_one(ESL_RANDOMNESS *r, ESL_DSQ oldc, ESL_DSQ newc, int L, ESL_DSQ *new, int *usecol, char *errbuf);
 
+int 
+msamanip_CalculateCTList(ESL_MSA *msa, CTLIST **ret_ctlist, int *ret_nbpairs, char *errbuf, int verbose)
+{
+  char   *tag = NULL;
+  CTLIST *ctlist = NULL;
+  int     L = msa->alen;
+  int     nbpairs = 0;
+  int     nct = 0;
+  int     gc;
+  int     c;
+  int     i, j;
+  int     status;
+
+  if (msa == NULL) return eslOK;
+
+  if (msa->ss_cons) {
+    ctlist = struct_wuss2CTList(msa->ss_cons, msa->alen, errbuf, verbose);
+    if (!ctlist)  ESL_FAIL(eslFAIL, errbuf, "bad SS_cons %s", msa->ss_cons);
+    nct    = ctlist->nct;
+   }
+ 
+  // create a ct for each GC SS_cons_x
+  esl_sprintf(&tag, "SS_cons_");
+  for (gc = 0; gc < msa->ngc; gc ++) {  
+
+    if (!strncmp(msa->gc_tag[gc], tag, 8)) {
+      if (ctlist) struct_ctlist_Realloc(ctlist, nct+1);
+      else        ctlist = struct_ctlist_Create(nct+1, L);
+
+      esl_wuss2ct(msa->gc[gc], L, ctlist->ct[nct]);
+      nct ++;
+    }
+  }
+
+  if (!ctlist) {
+    ctlist = struct_ctlist_Create(nct+1, L);
+    esl_vec_ISet(ctlist->ct[nct], msa->alen+1, 0);
+  }
+  
+  for  (c = 0; c < ctlist->nct; c ++) {
+    for (i = 0; i < msa->alen-1; i ++)
+      for (j = i+1; j < msa->alen; j ++)
+	if (ctlist->ct[c][i+1] == j+1) nbpairs ++;
+  }
+
+  if (ret_ctlist)  *ret_ctlist  = ctlist;  else struct_ctlist_Destroy(ctlist);
+  if (ret_nbpairs) *ret_nbpairs = nbpairs;
+
+  free(tag); 
+  return eslOK;
+
+ ERROR:
+  if (ctlist) struct_ctlist_Destroy(ctlist);
+  if (tag) free(tag);
+  return status;
+}
+
 int 
 msamanip_CalculateCT(ESL_MSA *msa, int **ret_ct, int *ret_nbpairs, double maxnowc, char *errbuf)
 {
@@ -147,6 +208,8 @@ msamanip_CalculateBC(ESL_MSA *msa, int *ct, double **ret_ft, double **ret_fbp, d
   return status;
 }
 
+
+
 int 
 msamanip_CompareBasecomp(ESL_MSA *msa1, ESL_MSA *msa2, char *errbuf)
 {
@@ -264,6 +327,56 @@ msamanip_ConvertMissingNonresidue2Gap(ESL_MSA *msa)
   return eslOK;
 }
 
+// check the msa name does not include |, replace with _
+// both FastTree and R2R have isssues with | in the msa name
+//
+// check for sequence names that contain a parenthesis () or []
+// replace with curly brakckets
+int 
+msamanip_DoctorSeqNames(const ESL_MSA *msa, char *errbuf)
+{
+  char *name;
+  int   n;
+  int   found = FALSE;
+  int   i;
+
+  for (n = 0; n < msa->nseq; n ++) {
+    if (strchr(msa->sqname[n], ')') || strchr(msa->sqname[n], '(') ||
+	strchr(msa->sqname[n], ']') || strchr(msa->sqname[n], '[')   )
+      {
+	name = msa->sqname[n];
+
+	i     = 0;
+	found = TRUE;
+	while (name[i] != '\0')
+	  {
+	    if (name[i] == '(' || name[i] == '[') name[i] = '{';
+	    if (name[i] == ')' || name[i] == ']') name[i] = '}';
+	    
+	    i++;
+	  }
+      }
+  }
+  if (found) printf("Warning: sequence names include parenthesis '(' | '[' | ')' | ']'. incompatible with the program FastTree. Replaced with '{' | '}'.\n\n");
+
+  return eslOK;
+}
+
+extern
+int msamanip_Getsqlen(ESL_MSA *msa)
+{
+  int n;
+  int status;
+  
+  ESL_ALLOC(msa->sqlen, sizeof(int64_t) * msa->nseq);  
+  for (n = 0; n < msa->nseq; n++) msa->sqlen[n] = esl_dsq_GetLen(msa->ax[n]);
+
+  return eslOK;
+
+ ERROR:
+  return status;
+}
+
 int
 msamanip_NonHomologous(ESL_ALPHABET *abc, ESL_MSA *msar, ESL_MSA *msae, int *ret_nhr, int *ret_nhe, int *ret_hr, int *ret_he, int *ret_hre, char *errbuf)
 {
@@ -351,9 +464,10 @@ msamanip_RemoveGapColumns(double gapthresh, ESL_MSA *msa, int64_t startpos, int6
   int     *map    = NULL;
   int     *revmap = NULL;
   int     *useme  = *ret_useme;
-  int      expgap;
+  double   idthresh = 1.0 - gapthresh;
+  double   r;
+  double   totwgt;
   int64_t  alen = msa->alen; //alen of the truncated alignment
-  int      ngap;
   int      apos;
   int      newpos = 0;
   int      i;
@@ -368,20 +482,28 @@ msamanip_RemoveGapColumns(double gapthresh, ESL_MSA *msa, int64_t startpos, int6
   }
  
   if (dofilter) {
+    // count the gaps/missing in apos
     for (apos = 1; apos <= alen; apos++) {
-      /* count the gaps/missing in apos */
-      ngap = 0;
-      for (i = 0; i < msa->nseq; i++) 
-	if (esl_abc_XIsGap(msa->abc, msa->ax[i][apos]) || esl_abc_XIsMissing(msa->abc, msa->ax[i][apos])) ngap ++;
+      r = totwgt = 0.;
+      for (i = 0; i < msa->nseq; i++)
+        {
+          if  (esl_abc_XIsResidue(msa->abc, msa->ax[i][apos]))
+	    {
+	      r += msa->wgt[i]; totwgt += msa->wgt[i];
+	    }
+          else if  (esl_abc_XIsGap(msa->abc,     msa->ax[i][apos])) totwgt += msa->wgt[i];
+          else if  (esl_abc_XIsMissing(msa->abc, msa->ax[i][apos]))  continue;
+        }
       
-      /* apply gapthresh */
-      expgap = ceil(gapthresh*(double)msa->nseq);
-      if (ngap > expgap) useme[apos-1] = FALSE;
+      // apply gapthresh 
+      if (r > 0. && r / totwgt >= idthresh) useme[apos-1] = TRUE;
+      else                                  useme[apos-1] = FALSE;
     }
     
-    if (msa->abc->type == eslRNA && (status = esl_msa_RemoveBrokenBasepairs(msa, errbuf, useme)) != eslOK)
+    if (msa->abc->type == eslRNA && (status = struct_RemoveBrokenBasepairs(msa, errbuf, useme)) != eslOK)
        ESL_XFAIL(eslFAIL, errbuf, "RemoveGapColumns(): error removing broken pairs");
-    if ((status = esl_msa_ColumnSubset         (msa, errbuf, useme)) != eslOK)
+    
+    if ((status = struct_ColumnSubset(msa, errbuf, useme)) != eslOK)
       ESL_XFAIL(eslFAIL, errbuf, "RemoveGapColumns(): error in esl_msa_ColumnSubset");
   }  
 
@@ -389,7 +511,8 @@ msamanip_RemoveGapColumns(double gapthresh, ESL_MSA *msa, int64_t startpos, int6
   ESL_ALLOC(map, sizeof(int) * msa->alen);
   for (apos = 0; apos < alen; apos++) 
     if (useme[apos]) map[newpos++] = apos + startpos;
-  if (newpos != msa->alen) ESL_XFAIL(eslFAIL, errbuf, "RemoveGapColumns(): truncation error tstart %d tend %d; is %d should be %d", startpos, endpos, newpos, msa->alen);
+  if (newpos != msa->alen)
+    ESL_XFAIL(eslFAIL, errbuf, "RemoveGapColumns(): truncation error tstart %d tend %d; is %d should be %d", startpos, endpos, newpos, msa->alen);
 
   if (ret_revmap) {
     newpos = 0;
@@ -427,6 +550,7 @@ msamanip_RemoveFragments(float fragfrac, ESL_MSA **msa, int *ret_nfrags, int *re
   double   clen = 0.;           // seq_cons length, otherwise avg lenght of sequences
   int64_t  alen;
   int64_t  len;
+  int64_t  L;
   int      n;
   int      i;
   int      x;
@@ -434,9 +558,14 @@ msamanip_RemoveFragments(float fragfrac, ESL_MSA **msa, int *ret_nfrags, int *re
 
   for (n = 0; n < omsa->ngc; n++) {
     if (strcmp(omsa->gc_tag[n], "seq_cons") == 0) {
-      esl_abc_CreateDsq(omsa->abc, omsa->gc[n], &dsq);
-      clen = (double)esl_abc_dsqrlen(omsa->abc, dsq);
-      alen = esl_abc_dsqlen(dsq);
+      ///esl_dsq_Create(omsa->abc, omsa->gc[n], &dsq);
+
+      L = strlen(omsa->gc[n]);
+      dsq = esl_dsq_Create(L);
+      esl_dsq_Digitize(omsa->abc, omsa->gc[n], dsq);
+
+      clen = (double)esl_dsq_GetRawLen(omsa->abc, dsq);
+      alen = esl_dsq_GetLen(dsq);
       break;
     }
   }
@@ -482,7 +611,7 @@ msamanip_RemoveFragments(float fragfrac, ESL_MSA **msa, int *ret_nfrags, int *re
   *ret_nfrags = omsa->nseq - esl_vec_ISum(useme, omsa->nseq);
 
   /* replace msa */
-  esl_msa_Destroy(omsa);
+  esl_msa_Destroy(*msa);
   *msa = new;
   
   if (dsq) free(dsq);
@@ -496,6 +625,42 @@ msamanip_RemoveFragments(float fragfrac, ESL_MSA **msa, int *ret_nfrags, int *re
   return status;
 }
 
+int
+msamanip_SingleSequenceRemoveGaps(ESL_MSA *msa, char *errbuf, int verbose)
+{
+  int     *useme = NULL;
+  int      s = 0;
+  int      l;
+  int      status;
+
+  if (msa->nseq > 1) return eslOK;
+  
+  // find the gaps (useme = FALSE)
+  ESL_ALLOC(useme, sizeof(int) * msa->alen);
+  esl_vec_ISet(useme, msa->alen, TRUE);
+
+  if (! (msa->flags & eslMSA_DIGITAL)) {
+    for (l = 0; l < msa->alen; l++)
+      if (esl_abc_CIsGap(msa->abc, msa->aseq[s][l])) useme[l] = FALSE;
+  }
+  else {
+    for (l = 0; l < msa->alen; l++)
+      if (esl_abc_XIsGap(msa->abc, msa->ax[s][l+1])) useme[l] = FALSE;
+  }
+
+  if ((status = esl_msa_ColumnSubset(msa, errbuf, useme)) != eslOK)
+    ESL_XFAIL(eslFAIL, errbuf, "Truncation failed\n");
+
+  if (verbose) esl_msafile_Write(stdout, msa, eslMSAFILE_STOCKHOLM);
+  
+  free(useme);
+  return eslOK;
+
+ ERROR:
+  if (useme) free(useme); 
+  return status;
+}
+
 int
 msamanip_Truncate(ESL_MSA *msa, int64_t tstart, int64_t tend, int64_t *ret_startpos, int64_t *ret_endpos, char *errbuf)
 {
@@ -722,7 +887,7 @@ msamanip_SelectSubsetByminID(ESL_RANDOMNESS *r, ESL_MSA **msa, float idthresh, i
 /* Extract subset with nseq sequences taken at random
  */
 int
-msamanip_SelectSubset(ESL_RANDOMNESS  *r, int nseq, ESL_MSA **omsa, char **msafile, char *errbuf, int verbose)
+msamanip_SelectSubset(ESL_RANDOMNESS  *r, int nseq, ESL_MSA **omsa, char **msafile, int randomize, char *errbuf, int verbose)
 {
   FILE         *msafp;
   char         *newfile = NULL;
@@ -743,7 +908,6 @@ msamanip_SelectSubset(ESL_RANDOMNESS  *r, int nseq, ESL_MSA **omsa, char **msafi
   }
   if (nseq > msa->nseq) ESL_XFAIL(eslFAIL, errbuf, "cannot sample %d sequences. Alignment has %d sequences.", nseq, msa->nseq);
   
-
   /* the newfile file with submsa */
   if (msafile) {
     omsafile = *msafile;
@@ -773,7 +937,9 @@ msamanip_SelectSubset(ESL_RANDOMNESS  *r, int nseq, ESL_MSA **omsa, char **msafi
   for (n = 0; n <= msa->nseq; n ++) array[n] = n;
 
   /* randomize array */
-  if ((status = esl_vec_IShuffle(r, array, msa->nseq)) != eslOK) ESL_XFAIL(status, errbuf, "failed to randomize array");
+  if (randomize) {
+    if ((status = esl_vec_IShuffle(r, array, msa->nseq)) != eslOK) ESL_XFAIL(status, errbuf, "failed to randomize array");
+  }
 
   /* get the first 'nseq' */
   ESL_ALLOC(useme, sizeof(int) * msa->nseq);
@@ -1280,7 +1446,8 @@ msamanip_ShuffleWithinRow(ESL_RANDOMNESS  *r, ESL_MSA *msa, ESL_MSA **ret_shmsa,
 }
 
 int 
-msamanip_ShuffleTreeSubstitutions(ESL_RANDOMNESS  *r, ESL_TREE *T, ESL_MSA *msa, ESL_MSA *allmsa, int *usecol, ESL_MSA **ret_shmsa, char *errbuf, int verbose)
+msamanip_ShuffleTreeSubstitutions(ESL_RANDOMNESS  *r, ESL_TREE *T, ESL_MSA *msa, ESL_MSA *allmsa, int *usecol, ESL_MSA **ret_shmsa,
+				  char *errbuf, int verbose)
 {
   ESL_MSA    *shallmsa = NULL;
   ESL_MSA    *shmsa = NULL;
@@ -1295,8 +1462,9 @@ msamanip_ShuffleTreeSubstitutions(ESL_RANDOMNESS  *r, ESL_TREE *T, ESL_MSA *msa,
   if (shallmsa == NULL) ESL_XFAIL(eslFAIL, errbuf, "bad shuffled msa");
 
   if (verbose) {
-      esl_msafile_Write(stdout, shallmsa, eslMSAFILE_STOCKHOLM); 
-    }
+    printf("sh_allmsa\n");
+    esl_msafile_Write(stdout, shallmsa, eslMSAFILE_STOCKHOLM); 
+  }
 
   /* vector to mark residues in a column */
   ESL_ALLOC(useme, sizeof(int) * allmsa->nseq);
@@ -1335,7 +1503,11 @@ msamanip_ShuffleTreeSubstitutions(ESL_RANDOMNESS  *r, ESL_TREE *T, ESL_MSA *msa,
       if (status != eslOK) ESL_XFAIL(eslFAIL, errbuf, "%s. Error in ShuffleTreeSubstitutions - right descendant", errbuf);
     }
   }
-  
+
+  if (verbose) {
+    esl_msafile_Write(stdout, shallmsa, eslMSAFILE_STOCKHOLM); 
+  }
+
   status = esl_msa_SequenceSubset(shallmsa, useme, &shmsa);
   if (status != eslOK) ESL_XFAIL(eslFAIL, errbuf, "Error in ShuffleTreeSubstitutions - could not create leaves' msa");
   
@@ -1426,16 +1598,24 @@ shuffle_tree_substitutions(ESL_RANDOMNESS *r, int aidx, int didx, ESL_DSQ *axa,
 {
   ESL_DSQ      *axash = shallmsa->ax[aidx];
   ESL_DSQ      *dxash = shallmsa->ax[didx];
-  ESL_ALPHABET *abc = shallmsa->abc;
-  int           K = shallmsa->abc->K+1;
-  int           K2 = K*K;
-  int           L = shallmsa->alen;
+  ESL_ALPHABET *abc   = shallmsa->abc;
+  int          *nsub  = NULL;
+  int           L     = shallmsa->alen;
+  int           K     = shallmsa->abc->K+1;
+  int           K2    = K*K;
   int           n;
   int           nsubs = 0;
-  int          *nsub = NULL;
   int           status;
  
 #if 0
+  printf("\n");
+  for (n = 1; n <= L; n++) {
+    printf("%d",  axa[n]); 
+  }
+  printf("\n");
+  for (n = 1; n <= L; n++) {
+    printf("%d",  axd[n]); 
+  }
   printf("\n");
   for (n = 1; n <= L; n++) {
     printf("%d",  axash[n]); 
@@ -1452,20 +1632,22 @@ shuffle_tree_substitutions(ESL_RANDOMNESS *r, int aidx, int didx, ESL_DSQ *axa,
   esl_vec_ISet(nsub, K2, 0);
   
   for (n = 1; n <= L; n++) {
-    if (usecol[n] &&
-	( esl_abc_XIsCanonical(abc, axa[n]) || esl_abc_XIsGap(abc, axa[n])) && 
-	( esl_abc_XIsCanonical(abc, axd[n]) || esl_abc_XIsGap(abc, axd[n])) &&
-	axa[n] != axd[n])
+    if (usecol[n]                         &&
+	axa[n] != axd[n]                  &&
+	(esl_abc_XIsCanonical(abc, axa[n]) || esl_abc_XIsGap(abc, axa[n])) && 
+	(esl_abc_XIsCanonical(abc, axd[n]) || esl_abc_XIsGap(abc, axd[n])) 
+	)
       {
 	nsub[axa[n]*K+axd[n]] ++;
 	nsubs ++;
       }
+    // assign sh-descendant as the sh-ascendent
     dxash[n] = axash[n];
   }
   
 #if 0
   int x;
-  if (1||verbose) {
+  if (verbose) {
     printf("nsub %d\n", nsubs);
     for (x = 0; x < K; x ++)
       printf("%d %d %d %d %d\n", nsub[x*K+0], nsub[x*K+1], nsub[x*K+2], nsub[x*K+3], nsub[x*K+4]);
@@ -1515,7 +1697,7 @@ shuffle_tree_substitute_all(ESL_RANDOMNESS *r, int K, int *nsub, int L, ESL_DSQ
   int       *colidx = NULL;
   int       *perm   = NULL;
   int        ncol;
-  int        ns;
+  int        ns, ns_t;
   int        n;
   int        c;
   int        s;
@@ -1528,10 +1710,10 @@ shuffle_tree_substitute_all(ESL_RANDOMNESS *r, int K, int *nsub, int L, ESL_DSQ
 
   for (oldc = 0; oldc < K; oldc ++) {
     
-    ns = 0;
+    ns_t = 0;
     for (newc = 0; newc < K; newc++) 
-      ns += nsub[oldc*K+newc];
-    if (ns == 0) continue;
+      ns_t += nsub[oldc*K+newc];
+    if (ns_t == 0) continue;
     
     /* find all other positions with oldc in ax */
     esl_vec_ISet(useme, L+1, FALSE);
@@ -1542,7 +1724,8 @@ shuffle_tree_substitute_all(ESL_RANDOMNESS *r, int K, int *nsub, int L, ESL_DSQ
 	ncol ++;
       }
     }
-    
+    if (ncol == 0) continue;
+
     ESL_ALLOC(colidx, sizeof(int) * ncol);
     ESL_ALLOC(perm,   sizeof(int) * ncol);
     c = 0;
@@ -1552,10 +1735,11 @@ shuffle_tree_substitute_all(ESL_RANDOMNESS *r, int K, int *nsub, int L, ESL_DSQ
     
     /* pick ns position to change */
     idx = ncol-1;
+    ns  = ns_t;
     for (newc = 0; newc < K; newc++) {
       s = nsub[oldc*K+newc];
 
-      while (s > 0) {
+      while (s > 0 && idx >= 0) {  // we have only ncol positions to substitute
 #if 0
 	printf("old %d new %d | ncol %d s %d ns %d idx %d colidx %d val %d\n", oldc, newc, ncol, s, ns, idx,  colidx[perm[idx]], ax[colidx[perm[idx]]]);
 	for (n = 1; n <= L; n++) {
@@ -1570,7 +1754,7 @@ shuffle_tree_substitute_all(ESL_RANDOMNESS *r, int K, int *nsub, int L, ESL_DSQ
 	ns  --;
       }
     }
-    if (ns > 0) ESL_XFAIL(eslFAIL, errbuf, "ns is %d should be zero", ns);
+    if (ns_t <= ncol && ns > 0) ESL_XFAIL(eslFAIL, errbuf, "ns is %d should be zero", ns);
 
 #if 0
     for (n = 1; n <= L; n++) {
@@ -1579,12 +1763,12 @@ shuffle_tree_substitute_all(ESL_RANDOMNESS *r, int K, int *nsub, int L, ESL_DSQ
     printf("\n");
 #endif
 
-    free(perm); perm = NULL;
+    free(perm);   perm   = NULL;
     free(colidx); colidx = NULL;
   }
 
   free(useme);
-  if (perm) free(perm);
+  if (perm)   free(perm);
   if (colidx) free(colidx);
   return eslOK;
 
